/*!
  \file Cfg.h

  \brief Library compile-time configuration. 
  
  OS, Debug mode selection
  
  $Id: Cfg.h,v 1.12 2006/05/27 08:55:01 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   19.07.2005 22:55:29 MSD
*/

#ifndef SKYPE_CFG_H
#define SKYPE_CFG_H

/// Target operation system type
typedef enum { os_win, os_nix, os_proxy } OS_t;

#if defined(USE_PROXY)
const OS_t OS(os_proxy);
#endif 

#if defined(__unix__) && (!defined(__CYGWIN__)) 
/// Target operation system. Compiler constant.
#ifndef USE_PROXY
const OS_t OS(os_nix); 
#endif
/// Target operation system. Preprocessor macro 
#define SKYPE_NIX
#endif 

#if defined(__MINGW32__)  
/// Target operation system. Compiler constant.
#ifndef USE_PROXY
const OS_t OS(os_win);
#endif
/// Target operation system. Preprocessor macro 
#define SKYPE_MINGW
#endif // __MINGW32__

#if defined(_MSC_VER) 
/// Target operation system. Compiler constant.
#ifndef USE_PROXY
const OS_t OS(os_win);
#endif
/// Target operation system. Preprocessor macro 
#define SKYPE_MSC
/// MSVC calls vsnprintf as _vsnprintf.  Fix.
#include <stdio.h>
#include <stdarg.h>
#ifndef vsnprintf
#define vsnprintf(a, b, c, d) _vsnprintf((a), (b), (c), (d))
#endif // vsnprintf

// C++ exception specification ignored except to indicate a function is not __declspec(nothrow)
#pragma warning( disable : 4290 )
// 'this' : used in base member initializer list
#pragma warning( disable : 4355 )
// 'conversion' conversion from 'type1' to 'type2', possible loss of data
#pragma warning( disable : 4244 )
// '=' : conversion from 'size_t' to 'DWORD', possible loss of data
#pragma warning( disable : 4267 )
#endif // _MSC_VER

/// Debug mode: debug or release version of library we should produce
typedef enum { debug, release } DbgMode_t;

#ifndef NDEBUG
/// Current debug mode.  This constant can be used in template parameters
const DbgMode_t DbgMode(debug);
#else
const DbgMode_t DbgMode(release);
#endif

#endif // SKYPE_CFG_H
