#include <boost/test/unit_test.hpp>
#include <boost/test/unit_test_suite.hpp>

#include "h/Application.h"

#include "SkypeTest.h"

using namespace boost::unit_test;
using namespace Ice;

/// Global variables 
SkypeConnection* SC;
std::string app_name("testapp");
std::string skype_name;
Application* APP;
bool isClient(false);

//const boost::posix_time::seconds tm(2);

std::string password = "This is test message";

void DefaultCallback(const std::string& s)
{
  std::cout << __FUNCTION__ << " : " << s.c_str() << std::endl;	
};

void OnCreateStream(AppStream& s)
{
  std::cout << __FUNCTION__ << " : New stream created : " << s.Name().c_str() << std::endl;	
}


void InitApp()
{
  const boost::posix_time::seconds tm(2); 
  SkypeConnection& sc = *SC;
  try
    {
      APP = new Application(sc,app_name,sc.GetLog());	  
      Application& app = *APP;
      app.SetCbk(OnCreateStream);
      ProcessSkypeMsgs(sc, tm);
      /// Create Application (skype object)
      app.Create();
      /// Wait for creation completion
      while (!app.IsCreated())
       { 
         ProcessSkypeMsgs(sc, tm);
       }
      /// Connect to skype name
      if(isClient) app.Connect(skype_name,OnCreateStream);
      /// Request attr "CONNECTABLE"
      app.AskAttr(apConnectable);
      ProcessSkypeMsgs(sc, tm);
      /// Request attr "CONNECTING"
      app.AskAttr(apConnecting);
      ProcessSkypeMsgs(sc, tm);
      /// Get application name (ID)
      std::string c = app.GetId();
      /// Show results
      std::cout << "Application name is " << c.c_str() << std::endl;
      c = app[apConnectable];
      std::cout << "Connectable users are " << c.c_str() << std::endl;
      c = app[apConnecting];
      std::cout << "Connecting users are " << c.c_str() << std::endl;
    }
  catch(std::exception& e)
    {
      std::cout << "ERROR " << e.what() << std::endl;
      abort();
    }
  catch(...)
    {
      std::cout << "UNKNOWN ERROR" << std::endl;
      abort();
    }
}


void WriteMsg()
{
  /// Init	
  const boost::posix_time::seconds tm(2);
  SkypeConnection& sc = *SC;
  assert(APP);
  Application& app = *APP;
  try
    {
      assert(app.size() && "No streams!");
      /// Send message to all application streams
      for(Application::const_iterator i = app.begin(); i!= app.end();i++)
        {
	  /// Don't forget AppFlush!	
	  *(i->second) << password << AppFlush();
          ProcessSkypeMsgs(sc, tm);
	}
      std::cout << "***************************************************" << std::endl;
      std::cout << "***        Client was finished successfully     ***" << std::endl;
      std::cout << "***************************************************" << std::endl;
      exit(0);
    }
  catch(std::exception& e)
    {
      std::cout << "ERROR " << e.what() << std::endl;
    }
  catch(...)
    {
      std::cout << "UNKNOWN ERROR" << std::endl;
    }
};

void ReadMsg()
{
  /// Init	
  const boost::posix_time::seconds tm(2);
  SkypeConnection& sc = *SC;
  assert(APP);
  Application& app = *APP;
  const boost::posix_time::millisec ms(100);
  /// Process S.M.
  ProcessSkypeMsgs(sc, tm);
  Impl::Sleep(ms);
  /// For each stream	    
  for(Application::const_iterator i = app.begin(); i!= app.end();i++)
    {
      /// Check for data	
      if(i->second->CanRead())
        {
          std::string tmp;
	  /// Read
          *(i->second) >> tmp;
	  /// Check password
	  if(tmp == password)
	    {
              std::cout << "***************************************************" << std::endl;
              std::cout << "***        Server was finished successfully     ***" << std::endl;
              std::cout << "***************************************************" << std::endl;
	      exit(0);
	    }
	  else
	    {
              std::cout << "***************************************************" << std::endl;
              std::cout << "***        Server error : incorrect message     ***" << std::endl;
              std::cout << "***************************************************" << std::endl;
	      exit(-1);
	    }
        }	      
    }	    
};


void AppTest()
{
    InitApp();
    if(isClient) WriteMsg();
    else while(1) ReadMsg();
};

test_suite* init_unit_test_suite(int argc, char* argv[])
{
  /// Check arguments	
  bool client = (argc == 2);
  bool server = (argc == 1);
  if (!client && !server) 
  {
	std::cout << "Usage: " << std::string(argv[0]) << " <skype name> for client mode" << std::endl;
    std::cout << "       " << std::string(argv[0]) << "              for server mode" << std::endl;
	exit(0);
  } else if(client) isClient = true;

  if(isClient) skype_name = argv[1];

  /// App logger	
  static FileLogger<> fl(stderr, LlDbg);
  static SimpleHandler_t def(DefaultCallback);
  
  const boost::posix_time::seconds tm(2);
  /// Skype connection
  static SkypeConnection scl(argv[0], def, LogFunc_t(&fl, &FileLogger<>::operator()));
  /// Attach
  try
  {
    scl.Attach();
    ProcessSkypeMsgs(scl, tm);
  }
  catch(std::exception& e)
  {
    std::cout << "ERROR : " << e.what() << std::endl;
    assert(0 && "Can't attach to skype");
  }
  catch(...)
  {
    assert(0 && "Can't attach to skype");
  }
  /// Last protocol version
  scl << DisableIds << "PROTOCOL 99" << flush << EnableIds;
  SC = &scl;
  
  test_suite* test= BOOST_TEST_SUITE("Application test");
  test->add(BOOST_TEST_CASE(&AppTest));
  return test;  
};
