/*!
  \file RawSkypeConnect.cpp

  \brief Test of RawSkypeConnection class
  
  $Id: RawSkypeConnect.cpp,v 1.3 2005/09/26 10:20:59 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   21.07.2005 23:32:19 MSD
*/

#include "SkypeTest.h"

#include "h/LowLevel.h"

#include <boost/test/unit_test.hpp>

using namespace boost::unit_test;

RawSkypeConnection<>* tst;
const char* prog_name;

void CallBack(const std::string& s)
{

  fprintf(stdout, "Callback is executed with data: '%s'\n", s.c_str());
//  if(!s.compare("PONG")) *tst << "PING" << flush;
}

void DoTest()
{	
  try {
	fprintf(stdout, "Testing of RawSkypeConnection class...\n");  
	RawNotify_t ntf(CallBack);
	tst = new RawSkypeConnection<>(ntf, prog_name);
	RawSkypeConnection<>& tst_r = *tst;
	tst_r.Attach();
	fprintf(stdout, "Successfully attached to skype\n");
	tst_r << DisableIds << "PROTOCOL " << 10 << flush;
	tst_r << EnableIds << "PING" << flush;
	tst_r << "GET SKYPEVERSION" << flush;
    
	const boost::posix_time::seconds tm(1);
	#if defined(SKYPE_MINGW) || defined(SKYPE_MSC)
		MSG oMessage;
    #endif // SKYPE_MINGW || SKYPE_MSC
	while(1) {
		#if defined(SKYPE_MINGW) || defined(SKYPE_MSC)
			if(PeekMessage(&oMessage, 0, 0, 0, PM_REMOVE) == 0) continue;
			TranslateMessage(&oMessage);
			DispatchMessage(&oMessage);
 		#endif // SKYPE_MINGW || SKYPE_MSC 
		tst_r();
		Impl::Sleep(tm); 
	};	
//	tst << "test" << flush;
  } catch(AssertionViolation& e) {
	fprintf(stdout, "AccessViolation error was catched: '%s'\n", e.what());	  
  } catch(SkypeDisconnect& e) {
	fprintf(stdout, "SkypeDisconnect error was catched: '%s'\n", e.what());  
  } catch(SkypeError& e) {
  	fprintf(stdout, "SkypeError was catched: '%s'\n", e.what());
  };	  
  delete tst;
};

test_suite* init_unit_test_suite( int argc, char* argv[] )
{
    prog_name = argv[0];	
    test_suite* test= BOOST_TEST_SUITE("RawSkypeConnection test");
    test->add(BOOST_TEST_CASE(&DoTest));
    
    return test;
};

